'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _immutable = require('immutable');

var _Track = require('./Track');

var _Track2 = _interopRequireDefault(_Track);

var _Step = require('./Step');

var _Step2 = _interopRequireDefault(_Step);

var _Handle = require('./Handle');

var _Handle2 = _interopRequireDefault(_Handle);

var _Range = require('./Range');

var _Range2 = _interopRequireDefault(_Range);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var defaultHandle = {
  index: 0,
  id: null,
  value: 1, // should be same as default props value
  label: null,
  position: 0,
  left: 0,
  top: 0,
  x: 0, // step center x
  y: 0, // step center y
  className: null
};

var defaultStep = {
  id: null,
  index: 0,
  label: null,
  value: 0,
  position: 0,
  left: 0,
  top: 0,
  x: 0, // step center x
  y: 0, // step center y
  inRange: null, // index of the range this step belongs to
  className: null
};

var defaultRange = {
  id: null,
  index: 0,
  label: null,
  fromHandle: -1, // by default from left border to first handle
  className: null,
  includeHandles: false,
  position: 0,
  length: 0,
  left: 0,
  top: 0,
  x: 0,
  y: 0,
  width: 0,
  height: 0
};

// default for this.state
var initialState = {
  container: {
    left: 0,
    top: 0,
    width: 0,
    height: 0
  },
  handles: [defaultHandle],
  handleLength: 0,
  handleGauge: 0,
  steps: [defaultStep],
  stepLength: 0,
  stepGauge: 0,
  ranges: [],
  track: {
    left: 0, // track top left corner num pixels from window left border
    top: 0, // track top left corner num pixels from window top border
    length: 0, // track length (independent of orientation)
    gauge: 0 },
  isHandleMoving: false,
  movingHandleIndex: 0,
  movingHandlePosition: 0,
  detailedValue: false
};

/**
 * Helper for preparing step objects
 */
var prepareSteps = function prepareSteps(props) {
  // Create steps if they are defined
  var steps = [];
  if (props.steps && props.steps.constructor === Array) {
    // use steps array provided as a prop
    steps = props.steps.map(function (step, i) {
      return Object.assign({}, defaultStep, {
        id: step.id ? step.id : i,
        index: i
      }, step);
    });
  } else if (props.steps) {
    // create steps based on min, max and step values
    for (var i = props.min; i <= props.max; i += props.step) {
      steps.push(Object.assign({}, defaultStep, {
        id: i,
        index: i,
        value: i,
        label: i.toString()
      }));
    }
  }
  return steps;
};

/**
 * Helper for preparing handle objects
 */
var prepareHandles = function prepareHandles(props) {
  // Create handles
  var handles = [defaultHandle];
  if (props.handles && props.handles.length > 0) {
    handles = props.handles.map(function (data, i) {
      return Object.assign({}, defaultHandle, data, { index: i });
    });
  } else if (props.value) {
    handles[0].value = props.value;
  }
  return handles;
};

/**
 * Helper for preparing range objects
 */
var prepareRanges = function prepareRanges(props) {
  var ranges = [];
  if (props.ranges && props.ranges.length > 0) {
    ranges = props.ranges.map(function (data, i) {
      return Object.assign({}, defaultRange, data, { index: i });
    });
  } else if (props.range) {
    if (props.range === true || props.range === 'below') {
      ranges.push(Object.assign({}, defaultRange));
    } else if (props.range === 'above') {
      ranges.push(Object.assign({}, defaultRange, { id: 0, fromHandle: 0 }));
    }
  }
  return ranges;
};

var decimalPlaces = function decimalPlaces(number) {
  var match = ('' + number).match(/(?:\.(\d+))?(?:[eE]([+-]?\d+))?$/);
  if (!match) {
    return 0;
  }
  return Math.max(0,
  // Number of digits right of decimal point.
  (match[1] ? match[1].length : 0) - (
  // Adjust for scientific notation.
  match[2] ? +match[2] : 0));
};

var calculateIfStepInRange = function calculateIfStepInRange(stepIndex, map) {
  var step = map.getIn(['steps', stepIndex]);
  var stepCenter = step.get('position');
  // Check if steps belongs to a range
  var inRange = null;
  map.get('ranges').map(function (range) {
    var rangePosition = range.get('position');
    var rangeLength = range.get('length');
    // const rangeCenterEnd = rangeLength - rangeCenterStart;
    if (stepCenter >= rangePosition && stepCenter <= rangeLength + rangePosition) {
      inRange = range.get('index');
    }
    return null;
  });
  return step.set('inRange', inRange);
};

var mapPropsToLocalState = function mapPropsToLocalState(props) {
  var map = (0, _immutable.fromJS)(initialState);
  // Prepare steps, handles
  var steps = prepareSteps({ steps: props.steps });
  map = map.set('steps', (0, _immutable.fromJS)(steps));
  var handles = prepareHandles({ handles: props.handles, value: props.value });
  map = map.set('handles', (0, _immutable.fromJS)(handles));
  var ranges = prepareRanges({ ranges: props.ranges, range: props.range });
  map = map.set('ranges', (0, _immutable.fromJS)(ranges));
  return map;
};

var SliderInput = function (_Component) {
  _inherits(SliderInput, _Component);

  function SliderInput(props) {
    _classCallCheck(this, SliderInput);

    var _this = _possibleConstructorReturn(this, (SliderInput.__proto__ || Object.getPrototypeOf(SliderInput)).call(this, props));

    _this.onHandleCatch = _this.onHandleCatch.bind(_this);
    _this.onHandleMove = _this.onHandleMove.bind(_this);
    _this.onHandleRelease = _this.onHandleRelease.bind(_this);
    _this.onTrackClick = _this.onTrackClick.bind(_this);
    _this.onRangeClick = _this.onRangeClick.bind(_this);
    _this.onStepClick = _this.onStepClick.bind(_this);
    _this.onWindowResize = _this.onWindowResize.bind(_this);
    _this.onWindowScroll = _this.onWindowScroll.bind(_this);
    _this.container = null; // ref (of root container element)
    _this.track = null; // ref
    _this.ranges = []; // refs array
    _this.steps = []; // refs array
    _this.handles = []; // refs array
    // create initial state
    _this.state = {
      map: mapPropsToLocalState({
        handles: props.handles,
        value: props.value,
        ranges: props.ranges,
        range: props.range,
        steps: props.steps
      })
    };
    return _this;
  }

  /* global window:false */


  _createClass(SliderInput, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      window.addEventListener('resize', this.onWindowResize);
      window.addEventListener('scroll', this.onWindowScroll);
      var map = this.calculate(this.state.map);
      this.setState({ map: map }); // eslint-disable-line
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(newProps) {
      var map = mapPropsToLocalState(newProps);
      map = this.calculate(map);
      this.setState({ map: map });
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      window.removeEventListener('resize', this.onWindowResize);
      window.removeEventListener('scroll', this.onWindowScroll);
    }
  }, {
    key: 'onWindowResize',
    value: function onWindowResize() {
      var map = this.calculate(this.state.map);
      this.setState({ map: map });
    }
  }, {
    key: 'onWindowScroll',
    value: function onWindowScroll() {
      var map = this.state.map;
      map = this.calculate(map);
      this.setState({ map: map });
    }
  }, {
    key: 'onHandleCatch',
    value: function onHandleCatch(index, eventX, eventY, deltaX, deltaY) {
      if (this.props.disabled) return;
      this.updateMovingHandlePositionInState(index, eventX, eventY, deltaX, deltaY);
    }
  }, {
    key: 'onHandleMove',
    value: function onHandleMove(index, eventX, eventY, deltaX, deltaY) {
      if (this.props.disabled) return;
      this.updateMovingHandlePositionInState(index, eventX, eventY, deltaX, deltaY);
    }
  }, {
    key: 'onHandleRelease',
    value: function onHandleRelease(index) {
      var map = this.state.map;
      // Reset state
      map = map.withMutations(function (mp) {
        mp.set('isHandleMoving', false).set('movingHandleIndex', -1).set('movingHandlePosition', null);
      });
      map = this.calculateMoving(map, index);
      this.triggerCallback(map, this.props.onChange);
      this.setState({ map: map });
    }
  }, {
    key: 'onTrackClick',
    value: function onTrackClick(eventX, eventY) {
      if (this.props.disabled) return;
      var map = this.state.map;
      var trackStart = this.vertical() ? this.state.map.getIn(['track', 'top']) : this.state.map.getIn(['track', 'left']);
      // const trackLength = map.getIn(['track', 'length']);
      var eventPos = this.vertical() ? eventY : eventX;
      var newPosition = eventPos - trackStart;

      var closestHandle = this.findClosest(map.get('handles'), newPosition);
      if (map.get('steps').size > 0) {
        var closestStep = this.findClosest(map.get('steps'), newPosition);
        map = map.setIn(['handles', closestHandle.get('index'), 'value'], closestStep.get('value'));
        map = map.setIn(['handles', closestHandle.get('index'), 'position'], closestStep.get('position'));
      } else {
        var closestStepValue = this.calculateClosestValue(newPosition);
        map = map.setIn(['handles', closestHandle.get('index'), 'value'], closestStepValue);
        map = map.setIn(['handles', closestHandle.get('index'), 'position'], newPosition);
      }
      map = this.calculateMoving(map);
      this.triggerCallback(map, this.props.onChange);
      this.setState({ map: map });
    }
  }, {
    key: 'onRangeClick',
    value: function onRangeClick(eventX, eventY) {
      this.onTrackClick(eventX, eventY);
    }
  }, {
    key: 'onStepClick',
    value: function onStepClick(stepIndex) {
      if (this.props.disabled) return;
      var map = this.state.map;
      var step = map.getIn(['steps', stepIndex]);
      var stepPos = this.vertical() ? step.get('y') : step.get('x');
      var closestHandle = this.findClosest(map.get('handles'), stepPos);
      map = map.setIn(['handles', closestHandle.get('index'), 'value'], step.get('value'));

      map = this.calculateMoving(map);

      this.triggerCallback(map, this.props.onChange);
      this.setState({ map: map });
    }
  }, {
    key: 'calculateClosestValue',
    value: function calculateClosestValue(position) {
      var map = this.state.map;
      var trackLength = map.getIn(['track', 'length']);
      var accurateValue = position / trackLength * (this.props.max - this.props.min) + this.props.min;
      var closestDistance = -1;
      var closestStepValue = null;
      var stepValue = this.props.min;
      var numDecimalPlaces = decimalPlaces(this.props.step);

      while (stepValue <= this.props.max) {
        var distance = Math.abs(stepValue - accurateValue);
        if (closestDistance < 0 || distance < closestDistance) {
          closestDistance = distance;
          closestStepValue = stepValue;
        }
        stepValue += this.props.step;
      }
      return closestStepValue.toFixed(numDecimalPlaces);
    }
  }, {
    key: 'calculateClosestPosition',
    value: function calculateClosestPosition(value, map) {
      var trackLength = map.getIn(['track', 'length']);
      var numDecimalPlaces = decimalPlaces(this.props.step);
      var position = (value - this.props.min) / (this.props.max - this.props.min) * trackLength;
      return position.toFixed(numDecimalPlaces);
    }
  }, {
    key: 'calculate',
    value: function calculate(map) {
      var _this2 = this;

      var m = map;
      // Container size
      var containerRect = this.container.getBoundingClientRect();
      m = m.set('container', (0, _immutable.fromJS)({
        left: containerRect.left,
        top: containerRect.top,
        width: containerRect.width,
        height: containerRect.height
      }));
      // Track dimensions and position
      var trackRect = this.track.getElement().getBoundingClientRect();
      m = m.set('track', (0, _immutable.fromJS)({
        left: trackRect.left,
        top: trackRect.top,
        length: this.vertical() ? trackRect.height : trackRect.width,
        gauge: this.vertical() ? trackRect.width : trackRect.height
      }));
      // Step dimensions
      if (this.steps[0]) {
        var stepRect = this.steps[0].getElement().getBoundingClientRect();
        m = m.set('stepLength', this.vertical() ? stepRect.height : stepRect.width);
        m = m.set('stepGauge', this.vertical() ? stepRect.width : stepRect.height);
      }

      // Step positions
      var steps = m.get('steps').map(function (step, i) {
        return _this2.calculateStepPosition(i, m);
      });
      m = m.set('steps', steps);

      // Handle dimensions
      var handleRect = this.handles[0].getElement().getBoundingClientRect(); // There's always at least one handle
      m = m.set('handleLength', this.vertical() ? handleRect.height : handleRect.width);
      m = m.set('handleGauge', this.vertical() ? handleRect.width : handleRect.height);

      m = this.calculateMoving(m);
      // Return a map
      return m;
    }
  }, {
    key: 'calculateMoving',
    value: function calculateMoving(map, handleIndex) {
      var _this3 = this;

      var m = map;
      if (handleIndex) {
        // Update single handle position
        var handle = this.calculateHandlePosition(handleIndex, m);
        m = m.setIn(['handles', handleIndex], handle);
      } else {
        // Update all handle positions
        var handles = m.get('handles').map(function (handle, i) {
          return _this3.calculateHandlePosition(i, m);
        });
        m = m.set('handles', handles);
      }
      // Range positions
      var ranges = m.get('ranges').map(function (range, i) {
        return _this3.calculateRangePosition(i, m);
      });
      m = m.set('ranges', ranges);
      // Is step in range?
      var steps = m.get('steps').map(function (step, i) {
        return calculateIfStepInRange(i, m);
      });
      m = m.set('steps', steps);
      return m;
    }
  }, {
    key: 'calculateStepPosition',
    value: function calculateStepPosition(stepIndex, map) {
      var step = map.getIn(['steps', stepIndex]);
      var trackLength = map.getIn(['track', 'length']);
      var trackGauge = map.getIn(['track', 'gauge']);
      var stepLength = map.get('stepLength');
      var stepGauge = map.get('stepGauge');
      var trackStart = this.vertical() ? map.getIn(['track', 'top']) : map.getIn(['track', 'left']);
      var containerStart = this.vertical() ? map.getIn(['container', 'top']) : map.getIn(['container', 'left']);

      var trackPadding = trackStart - containerStart;

      var numSteps = map.get('steps') ? map.get('steps').size : 0;

      var stepCenterGauge = trackGauge / 2;
      var stepCenterLength = stepIndex * (trackLength / (numSteps - 1)); // steps over track endings

      var stepPositionGauge = stepCenterGauge - stepGauge / 2;
      var stepPositionLength = stepCenterLength + trackPadding - stepLength / 2;

      if (this.vertical()) {
        step = step.withMutations(function (st) {
          st.set('position', stepCenterLength).set('left', stepPositionGauge).set('top', stepPositionLength).set('x', stepCenterGauge).set('y', stepCenterLength);
        });
      } else {
        step = step.withMutations(function (st) {
          st.set('position', stepCenterLength).set('left', stepPositionLength).set('top', stepPositionGauge).set('x', stepCenterLength).set('y', stepCenterGauge);
        });
      }
      return step;
    }
  }, {
    key: 'calculateHandlePosition',
    value: function calculateHandlePosition(handleIndex, map) {
      var handle = map.getIn(['handles', handleIndex]);
      var trackLength = map.getIn(['track', 'length']);
      var trackGauge = map.getIn(['track', 'gauge']);
      var handleLength = map.get('handleLength');
      var handleGauge = map.get('handleGauge');
      var isHandleMoving = map.get('isHandleMoving');
      var movingHandleIndex = map.get('movingHandleIndex');
      var movingHandlePosition = map.get('movingHandlePosition');

      var trackStart = this.vertical() ? map.getIn(['track', 'top']) : map.getIn(['track', 'left']);
      var containerStart = this.vertical() ? map.getIn(['container', 'top']) : map.getIn(['container', 'left']);
      var trackPadding = trackStart - containerStart;

      var handleCenterLength = null;
      var steps = map.get('steps');

      if (isHandleMoving && movingHandleIndex === handleIndex) {
        handleCenterLength = movingHandlePosition - trackPadding;
        /*
        if (steps && steps.size > 0) {
          // Snap to steps, if steps are available
          const closestStep = this.findClosest(steps, movingHandlePosition);
          handleCenterLength = closestStep.get('position');
        }
        */
        // Prevent running over next handle
        if (map.hasIn(['handles', handleIndex + 1])) {
          var nextHandle = map.getIn(['handles', handleIndex + 1]);
          var nextHandlePosition = this.vertical() ? nextHandle.get('y') : nextHandle.get('x');
          if (movingHandlePosition > nextHandlePosition - handleLength) {
            handleCenterLength = nextHandlePosition - handleLength;
          }
        }
        // Prevent running over previous handle
        if (handleIndex > 0 && map.hasIn(['handles', handleIndex - 1])) {
          var prevHandle = map.getIn(['handles', handleIndex - 1]);
          var prevHandlePosition = this.vertical() ? prevHandle.get('y') : prevHandle.get('x');
          if (movingHandlePosition < prevHandlePosition + handleLength) {
            handleCenterLength = prevHandlePosition + handleLength;
          }
        }
      } else if (steps && steps.size > 0) {
        // not moving, with steps
        // If there are steps.
        // Find matching step by value
        var closestStep = null;
        steps.map(function (step) {
          if (step.get('value') === handle.get('value')) {
            closestStep = step;
          }
          return null;
        });
        // TODO: Prevent dropping handle on the wrong side of next/prev handle
        if (closestStep) {
          handleCenterLength = closestStep.get('position');
        }
      } else {
        // not moving, no steps
        // No steps are used on track. Use value -> position conversion
        handleCenterLength = handle.get('position');
        if (handle.get('value')) {
          handleCenterLength = this.calculateClosestPosition(handle.get('value'), map);
        }
      }

      // Prevent going over borders
      if (handleCenterLength < 0) {
        handleCenterLength = 0;
      }
      if (handleCenterLength > trackLength) {
        handleCenterLength = trackLength;
      }

      var handleCenterGauge = trackGauge / 2;
      var handlePositionLength = handleCenterLength - handleLength / 2 + trackPadding;
      var handlePositionGauge = handleCenterGauge - handleGauge / 2;

      if (this.vertical()) {
        handle = handle.withMutations(function (st) {
          st.set('position', handleCenterLength).set('left', handlePositionGauge).set('top', handlePositionLength).set('x', handleCenterGauge).set('y', handleCenterLength);
        });
      } else {
        handle = handle.withMutations(function (st) {
          st.set('position', handleCenterLength).set('left', handlePositionLength).set('top', handlePositionGauge).set('x', handleCenterLength).set('y', handleCenterGauge);
        });
      }
      return handle;
    }
  }, {
    key: 'calculateRangePosition',
    value: function calculateRangePosition(rangeIndex, map) {
      var trackLength = map.getIn(['track', 'length']);
      var trackGauge = map.getIn(['track', 'gauge']);
      var handleLength = map.get('handleLength');

      var trackStart = this.vertical() ? map.getIn(['track', 'top']) : map.getIn(['track', 'left']);
      var containerStart = this.vertical() ? map.getIn(['container', 'top']) : map.getIn(['container', 'left']);
      var trackPadding = trackStart - containerStart;
      var stepLength = map.get('stepLength');

      var range = map.getIn(['ranges', rangeIndex]);
      var rangePositionStart = 0 - stepLength / 2;
      var rangePositionEnd = trackLength + stepLength / 2;
      // Find start and end handle
      var fromHandleIndex = range.get('fromHandle');
      if (fromHandleIndex > -1) {
        var fromHandle = map.getIn(['handles', fromHandleIndex]);
        rangePositionStart = fromHandle.get('position') + handleLength / 2;
        if (range.get('includeHandles')) {
          rangePositionStart -= handleLength;
        }
      }
      var toHandleIndex = 0;
      if (fromHandleIndex >= 0) {
        toHandleIndex = fromHandleIndex + 1;
      }
      var toHandle = map.getIn(['handles', toHandleIndex]);
      if (toHandle) {
        rangePositionEnd = toHandle.get('position') - handleLength / 2;
        if (range.get('includeHandles')) {
          rangePositionEnd += handleLength;
        }
      }
      var rangePosition = Math.floor(rangePositionStart + trackPadding);
      var rangeLength = Math.ceil(rangePositionEnd - rangePositionStart);

      if (this.vertical()) {
        range = range.withMutations(function (st) {
          st.set('position', rangePosition).set('length', rangeLength).set('left', 0).set('top', rangePosition).set('width', trackGauge).set('height', rangeLength);
        });
      } else {
        range = range.withMutations(function (st) {
          st.set('position', rangePosition).set('length', rangeLength).set('left', rangePosition).set('top', 0).set('width', rangeLength).set('height', trackGauge);
        });
      }
      return range;
    }

    /**
     * @param index of the handle
     * @param eventX position of the mouse/touch event from window left
     * @param eventY position of the mouse/touch event from window top
     * @param deltaX distance from click event x to handle's left border
     * @param deltaY distance from click event y to handle's top border
     */

  }, {
    key: 'updateMovingHandlePositionInState',
    value: function updateMovingHandlePositionInState(handleIndex, eventX, eventY, deltaX, deltaY) {
      var containerStart = this.vertical() ? this.state.map.getIn(['container', 'top']) : this.state.map.getIn(['container', 'left']);
      var map = this.state.map;
      // const trackStart = this.vertical() ? this.state.map.getIn(['track', 'top']) : this.state.map.getIn(['track', 'left']);
      var handleLength = map.get('handleLength');
      var eventPos = this.vertical() ? eventY : eventX;
      var delta = this.vertical() ? deltaY : deltaX;
      // const trackPadding = trackStart - containerStart;
      var handlePosition = eventPos - delta - containerStart + handleLength / 2;

      var trackLength = map.getIn(['track', 'length']);
      var numSteps = map.get('steps').size;
      var stepLength = trackLength / (numSteps - 1);

      var trackStart = this.vertical() ? map.getIn(['track', 'top']) : map.getIn(['track', 'left']);
      var trackPadding = trackStart - containerStart;

      // Prevent running over next handle
      if (map.hasIn(['handles', handleIndex + 1])) {
        var nextHandle = map.getIn(['handles', handleIndex + 1]);
        var nextHandlePosition = this.vertical() ? nextHandle.get('y') : nextHandle.get('x');
        if (handlePosition > nextHandlePosition - stepLength + trackPadding) {
          handlePosition = nextHandlePosition - stepLength + trackPadding;
        }
      }
      // Prevent running over previous handle
      if (handleIndex > 0 && map.hasIn(['handles', handleIndex - 1])) {
        var prevHandle = map.getIn(['handles', handleIndex - 1]);
        var prevHandlePosition = this.vertical() ? prevHandle.get('y') : prevHandle.get('x');
        if (handlePosition < prevHandlePosition + stepLength + trackPadding) {
          handlePosition = prevHandlePosition + stepLength + trackPadding;
        }
      }
      map = map.withMutations(function (mp) {
        mp.set('isHandleMoving', true).set('movingHandleIndex', handleIndex).set('movingHandlePosition', handlePosition);
      });
      // Calculate new value for handle by finding closest step to handle
      var movingHandlePosition = map.get('movingHandlePosition');
      if (map.get('steps').size > 0) {
        var closestStep = this.findClosest(map.get('steps'), movingHandlePosition - trackPadding);
        // Snap to closest step
        map = map.setIn(['handles', handleIndex, 'value'], closestStep.get('value'));
        map = map.setIn(['handles', handleIndex, 'position'], closestStep.get('position'));
      } else {
        var closestStepValue = this.calculateClosestValue(movingHandlePosition);
        map = map.setIn(['handles', handleIndex, 'value'], closestStepValue);
        map = map.setIn(['handles', handleIndex, 'position'], movingHandlePosition);
      }

      map = this.calculateMoving(map, handleIndex);
      this.triggerCallback(map, this.props.onMove);
      this.setState({ map: map });
    }

    /**
     * Notify 3rd party code by calling onChange
     */

  }, {
    key: 'triggerCallback',
    value: function triggerCallback(map, callback) {
      if (this.props.disabled) return;
      var trackLength = map.getIn(['track', 'length']);
      // Build value for 3rd party code
      var handles = map.get('handles');
      // RETURN SIMPLE VALUE
      if (!this.props.detailedValue && handles.size === 1) {
        if (callback) {
          callback(handles.getIn([0, 'value']));
        }
        return;
      }
      // RETURN COMPLEX VALUE
      var data = handles.reduce(function (result, handle, index) {
        // Find matching step
        var matchingStep = map.get('steps').find(function (step) {
          return step.get('value') === handle.get('value');
        });
        var item = {
          id: handle.get('id') || null,
          value: handle.get('value'),
          position: (handle.get('position') / trackLength).toFixed(3),
          step: {}
        };
        if (matchingStep) {
          item.step = {
            id: matchingStep.get('id'),
            index: matchingStep.get('index')
          };
        }
        return result.set(index, (0, _immutable.fromJS)(item));
      }, (0, _immutable.List)()).toList();
      if (callback) {
        callback(data.toJS());
      }
    }

    /**
     * Works for all Map items in List that have x and y as props
     * E.g. step, handle
     * @param List of Map objects with center  x and y coordinates
     */

  }, {
    key: 'findClosest',
    value: function findClosest(list, position) {
      var _this4 = this;

      var closestItem = null;
      var closestDistance = -1;
      list.map(function (item) {
        var itemPosition = _this4.vertical() ? item.get('y') : item.get('x');
        var distance = Math.abs(position - itemPosition);
        if (closestDistance < 0 || distance < closestDistance) {
          closestDistance = distance;
          closestItem = item;
        }
        return null;
      });
      return closestItem;
    }
  }, {
    key: 'vertical',
    value: function vertical() {
      return this.props.orientation === 'vertical';
    }
  }, {
    key: 'render',
    value: function render() {
      var _this5 = this;

      var classNames = 'ReactMotionSliderInput ' + (this.vertical() ? 'vertical' : 'horizontal') + (this.props.disabled ? ' disabled' : '') + (this.props.className ? ' ' + this.props.className : '');
      return _react2.default.createElement(
        'div',
        {
          className: classNames,
          ref: function ref(c) {
            _this5.container = c;
          }
        },
        _react2.default.createElement(
          _Track2.default,
          {
            ref: function ref(c) {
              _this5.track = c;
            },
            orientation: this.vertical() ? 'vertical' : 'horizontal',
            onClick: this.onTrackClick,
            handleLength: this.state.map.get('handleLength')
          },
          this.state.map.get('ranges').map(function (range, i) {
            return _react2.default.createElement(_Range2.default, {
              ref: function ref(c) {
                _this5.ranges[i] = c;
              },
              index: i,
              key: 'react-motion-input-slider-range-' + i,
              label: range.get('label'),
              value: range.get('value'),
              left: range.get('left'),
              top: range.get('top'),
              width: range.get('width'),
              height: range.get('height'),
              className: range.get('className'),
              spring: _this5.props.spring,
              onClick: _this5.onRangeClick
            });
          }),
          this.state.map.get('steps').map(function (step, i) {
            return _react2.default.createElement(_Step2.default, {
              ref: function ref(c) {
                _this5.steps[i] = c;
              },
              index: i,
              key: 'react-motion-input-slider-step-' + i,
              label: step.get('label'),
              value: step.get('value'),
              left: step.get('left'),
              top: step.get('top'),
              inRange: step.get('inRange'),
              className: step.get('className'),
              onClick: _this5.onStepClick
            });
          }),
          this.state.map.get('handles').map(function (handle, i) {
            return _react2.default.createElement(_Handle2.default, {
              ref: function ref(c) {
                _this5.handles[i] = c;
              },
              index: i,
              key: 'react-motion-input-slider-handle-' + i,
              left: handle.get('left'),
              top: handle.get('top'),
              onCatch: _this5.onHandleCatch,
              onMove: _this5.onHandleMove,
              onRelease: _this5.onHandleRelease,
              label: handle.get('label'),
              spring: _this5.props.spring
            });
          })
        )
      );
    }
  }]);

  return SliderInput;
}(_react.Component);

SliderInput.defaultProps = {
  orientation: 'horizontal',
  steps: null,
  value: 1,
  step: 1,
  min: 1,
  max: 10,
  range: false,
  handles: null,
  ranges: null,
  spring: {
    stiffness: 1000,
    damping: 40, // how much spring goes back and forth
    precision: 0.01
  }
};
SliderInput.propTypes = {
  orientation: _react.PropTypes.string,
  step: _react.PropTypes.number,
  min: _react.PropTypes.number,
  max: _react.PropTypes.number,
  // shorthand for setting a single handle with a value
  value: _react.PropTypes.oneOfType([_react.PropTypes.string, _react.PropTypes.number]),
  // Define steps manually
  steps: _react.PropTypes.oneOfType([_react.PropTypes.bool, _react.PropTypes.arrayOf(_react.PropTypes.shape({
    label: _react.PropTypes.string,
    value: _react.PropTypes.oneOfType([_react.PropTypes.string, _react.PropTypes.number])
  }))]),
  // Define handles manually (overrides 'value' shorthand if set)
  handles: _react.PropTypes.arrayOf(_react.PropTypes.shape({
    value: _react.PropTypes.oneOfType([_react.PropTypes.string, _react.PropTypes.number]).isRequired,
    id: _react.PropTypes.string,
    label: _react.PropTypes.string,
    className: _react.PropTypes.string
  })),
  // Define which ranges are visible between handles
  ranges: _react.PropTypes.arrayOf(_react.PropTypes.shape({
    id: _react.PropTypes.string,
    label: _react.PropTypes.string,
    fromHandle: _react.PropTypes.number, // index of lower handle
    className: _react.PropTypes.string,
    includeHandles: _react.PropTypes.bool
  })),
  // Shorthand for range props
  range: _react.PropTypes.oneOfType([_react.PropTypes.bool, // if true, set range below, false is noop
  _react.PropTypes.string]),
  onMove: _react.PropTypes.func,
  onChange: _react.PropTypes.func,
  className: _react.PropTypes.string,
  disabled: _react.PropTypes.bool,
  // React-motion spring props
  spring: _react.PropTypes.shape({
    stiffness: _react.PropTypes.number,
    damping: _react.PropTypes.number,
    precision: _react.PropTypes.number
  }),
  detailedValue: _react.PropTypes.bool
};
exports.default = SliderInput;